@@ -1,6 +1,6 @@
 /*
 32.xlatetof.c - implementation of the elf32_xlateto[fm](3) functions.
-Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 2002 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include <byteswap.h>
 
 #ifndef lint
-static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 michael Exp $";
+static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.17 2002/12/22 20:08:48 michael Exp $";
 #endif /* lint */
 
 /*
@@ -52,26 +52,32 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
  * function instantiator
  */
 #define copy_type_e_io(name,e,io,tfrom,tto,copy)		\
-    static void							\
+    static size_t						\
     Cat3(name,_,io)(unsigned char *dst, const unsigned char *src, size_t n) {	\
-	const tfrom *from = (const tfrom*)src;			\
-	tto *to = (tto*)dst;					\
-	if (sizeof(tfrom) < sizeof(tto)) {			\
-	    from += n;						\
-	    to += n;						\
-	    while (n-- > 0) {					\
-		--from;						\
-		--to;						\
-		copy(e,io,seq_back)				\
+	n /= sizeof(tfrom);					\
+	if (n && dst) {						\
+	    const tfrom *from = (const tfrom*)src;		\
+	    tto *to = (tto*)dst;				\
+	    size_t i;						\
+								\
+	    if (sizeof(tfrom) < sizeof(tto)) {			\
+		from += n;					\
+		to += n;					\
+		for (i = 0; i < n; i++) {			\
+		    --from;					\
+		    --to;					\
+		    copy(e,io,seq_back)				\
+		}						\
 	    }							\
-	}							\
-	else {							\
-	    while (n-- > 0) {					\
-		copy(e,io,seq_forw)				\
-		from++;						\
-		to++;						\
+	    else {						\
+		for (i = 0; i < n; i++) {			\
+		    copy(e,io,seq_forw)				\
+		    from++;					\
+		    to++;					\
+		}						\
 	    }							\
 	}							\
+	return n * sizeof(tto);					\
     }
 
 #define copy_type_e(name,e,type,copy)				\
@@ -126,7 +132,7 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
 #define copy_dyn_11(e,io,seq)		\
     seq(copy_sword(e,io,d_tag),		\
     seq(copy_addr(e,io,d_un.d_ptr),	\
-    /**/))
+    nullcopy))
 #define copy_ehdr_11(e,io,seq)		\
     seq(copy_arr(e,io,e_ident),		\
     seq(copy_half(e,io,e_type),		\
@@ -142,7 +148,7 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
     seq(copy_half(e,io,e_shentsize),	\
     seq(copy_half(e,io,e_shnum),	\
     seq(copy_half(e,io,e_shstrndx),	\
-    /**/))))))))))))))
+    nullcopy))))))))))))))
 #define copy_phdr_11(e,io,seq)		\
     seq(copy_word(e,io,p_type),		\
     seq(copy_off(e,io,p_offset),	\
@@ -152,16 +158,16 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
     seq(copy_word(e,io,p_memsz),	\
     seq(copy_word(e,io,p_flags),	\
     seq(copy_word(e,io,p_align),	\
-    /**/))))))))
+    nullcopy))))))))
 #define copy_rela_11(e,io,seq)		\
     seq(copy_addr(e,io,r_offset),	\
     seq(copy_word(e,io,r_info),		\
     seq(copy_sword(e,io,r_addend),	\
-    /**/)))
+    nullcopy)))
 #define copy_rel_11(e,io,seq)		\
     seq(copy_addr(e,io,r_offset),	\
     seq(copy_word(e,io,r_info),		\
-    /**/))
+    nullcopy))
 #define copy_shdr_11(e,io,seq)		\
     seq(copy_word(e,io,sh_name),	\
     seq(copy_word(e,io,sh_type),	\
@@ -173,7 +179,7 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
     seq(copy_word(e,io,sh_info),	\
     seq(copy_word(e,io,sh_addralign),	\
     seq(copy_word(e,io,sh_entsize),	\
-    /**/))))))))))
+    nullcopy))))))))))
 #define copy_sym_11(e,io,seq)		\
     seq(copy_word(e,io,st_name),	\
     seq(copy_addr(e,io,st_value),	\
@@ -181,29 +187,34 @@ static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 m
     seq(copy_byte(e,io,st_info),	\
     seq(copy_byte(e,io,st_other),	\
     seq(copy_half(e,io,st_shndx),	\
-    /**/))))))
+    nullcopy))))))
 
-static void
+#define nullcopy /**/
+
+static size_t
 byte_copy(unsigned char *dst, const unsigned char *src, size_t n) {
-    if (dst == src || !n) {
-	return;
-    }
+    if (n && dst && dst != src) {
 #if HAVE_BROKEN_MEMMOVE
-    while (dst > src && dst < &src[n]) {
-	if (n <= 16) {
-	    /* copy `manually' */
-	    while (n--) {
-		dst[n] = src[n];
-	    }
-	    return;
+	size_t i;
+
+	if (dst >= src + n || dst + n <= src) {
+	    memcpy(dst, src, n);
 	}
-	/* copy upper half */
-	byte_copy(&dst[n / 2], &src[n / 2], n - n / 2);
-	/* continue with lower half */
-	n /= 2;
-    }
+	else if (dst < src) {
+	    for (i = 0; i < n; i++) {
+		dst[i] = src[i];
+	    }
+	}
+	else {
+	    for (i = n; --i; ) {
+		dst[i] = src[i];
+	    }
+	}
+#else /* HAVE_BROKEN_MEMMOVE */
+	memmove(dst, src, n);
 #endif /* HAVE_BROKEN_MEMMOVE */
-    memmove(dst, src, n);
+    }
+    return n;
 }
 
 static void
@@ -230,16 +241,20 @@ copy_type(rel_32,11,Elf32_Rel,copy_rel_11)
 copy_type(shdr_32,11,Elf32_Shdr,copy_shdr_11)
 copy_type(sym_32,11,Elf32_Sym,copy_sym_11)
 
-typedef void (*xlator)(unsigned char*, const unsigned char*, size_t);
+typedef size_t (*xlator)(unsigned char*, const unsigned char*, size_t);
 typedef xlator xltab[ELF_T_NUM][2];
 
 /*
  * translation table (32-bit, version 1 -> version 1)
  */
+#if PIC
+static xltab
+#else /* PIC */
 static const xltab
+#endif /* PIC */
 xlate32_11[/*encoding*/] = {
     {
-	{ byte_copy,	    byte_copy	    },
+	{ byte_copy,        byte_copy       },
 	{ addr_32L__tom,    addr_32L__tof   },
 	{ dyn_32L11_tom,    dyn_32L11_tof   },
 	{ ehdr_32L11_tom,   ehdr_32L11_tof  },
@@ -252,9 +267,18 @@ xlate32_11[/*encoding*/] = {
 	{ sword_32L__tom,   sword_32L__tof  },
 	{ sym_32L11_tom,    sym_32L11_tof   },
 	{ word_32L__tom,    word_32L__tof   },
+	{ NULL,             NULL            },	/* there is no Sxword */
+	{ NULL,             NULL            },	/* there is no Xword */
+#if __LIBELF_SYMBOL_VERSIONS
+	{ _elf_verdef_32L11_tom,  _elf_verdef_32L11_tof  },
+	{ _elf_verneed_32L11_tom, _elf_verneed_32L11_tof },
+#else /* __LIBELF_SYMBOL_VERSIONS */
+	{ NULL,             NULL            },
+	{ NULL,             NULL            },
+#endif /* __LIBELF_SYMBOL_VERSIONS */
     },
     {
-	{ byte_copy,	    byte_copy	    },
+	{ byte_copy,        byte_copy       },
 	{ addr_32M__tom,    addr_32M__tof   },
 	{ dyn_32M11_tom,    dyn_32M11_tof   },
 	{ ehdr_32M11_tom,   ehdr_32M11_tof  },
@@ -267,13 +291,26 @@ xlate32_11[/*encoding*/] = {
 	{ sword_32M__tom,   sword_32M__tof  },
 	{ sym_32M11_tom,    sym_32M11_tof   },
 	{ word_32M__tom,    word_32M__tof   },
+	{ NULL,             NULL            },	/* there is no Sxword */
+	{ NULL,             NULL            },	/* there is no Xword */
+#if __LIBELF_SYMBOL_VERSIONS
+	{ _elf_verdef_32M11_tom,  _elf_verdef_32M11_tof  },
+	{ _elf_verneed_32M11_tom, _elf_verneed_32M11_tof },
+#else /* __LIBELF_SYMBOL_VERSIONS */
+	{ NULL,             NULL            },
+	{ NULL,             NULL            },
+#endif /* __LIBELF_SYMBOL_VERSIONS */
     },
 };
 
 /*
  * main translation table (32-bit)
  */
+#if PIC
+static xltab*
+#else /* PIC */
 static const xltab *const
+#endif /* PIC */
 xlate32[EV_CURRENT - EV_NONE][EV_CURRENT - EV_NONE] = {
     { xlate32_11, },
 };
@@ -285,14 +322,52 @@ xlate32[EV_CURRENT - EV_NONE][EV_CURRENT - EV_NONE] = {
 	    [(type) - ELF_T_BYTE]	\
 	    [d])
 
+/*
+ * destination buffer size
+ */
+size_t
+_elf32_xltsize(const Elf_Data *src, unsigned dv, unsigned encode, int tof) {
+    Elf_Type type = src->d_type;
+    unsigned sv = src->d_version;
+    xlator op;
+
+    if (!valid_version(sv) || !valid_version(dv)) {
+	seterr(ERROR_UNKNOWN_VERSION);
+	return (size_t)-1;
+    }
+    if (tof) {
+	/*
+	 * Encoding doesn't really matter (the translator only looks at
+	 * the source, which resides in memory), but we need a proper
+	 * encoding to select a translator...
+	 */
+	encode = ELFDATA2LSB;
+    }
+    else if (!valid_encoding(encode)) {
+	seterr(ERROR_UNKNOWN_ENCODING);
+	return (size_t)-1;
+    }
+    if (!valid_type(type)) {
+	seterr(ERROR_UNKNOWN_TYPE);
+	return (size_t)-1;
+    }
+    if (!(op = translator(sv, dv, encode, type, tof))) {
+	seterr(ERROR_UNKNOWN_TYPE);
+	return (size_t)-1;
+    }
+    return (*op)(NULL, src->d_buf, src->d_size);
+}
+
 /*
  * direction-independent translation
  */
 static Elf_Data*
 elf32_xlate(Elf_Data *dst, const Elf_Data *src, unsigned encode, int tof) {
-    size_t ssize, dsize, count;
     Elf_Type type;
-    int sv, dv;
+    int dv;
+    int sv;
+    size_t dsize;
+    size_t tmp;
     xlator op;
 
     if (!src || !dst) {
@@ -317,22 +392,21 @@ elf32_xlate(Elf_Data *dst, const Elf_Data *src, unsigned encode, int tof) {
 	seterr(ERROR_UNKNOWN_TYPE);
 	return NULL;
     }
-    ssize = _fmsize(ELFCLASS32, sv, type, 1 - tof);
-    dsize = _fmsize(ELFCLASS32, dv, type, tof);
     op = translator(sv, dv, encode, type, tof);
-    if (!ssize || !dsize || !op) {
+    if (!op) {
 	seterr(ERROR_UNKNOWN_TYPE);
 	return NULL;
     }
-    count = src->d_size / ssize;
-    if (dst->d_size < count * dsize) {
+    dsize = (*op)(NULL, src->d_buf, src->d_size);
+    if (dst->d_size < dsize) {
 	seterr(ERROR_DST2SMALL);
 	return NULL;
     }
-    if (count) {
-	(*op)(dst->d_buf, src->d_buf, count);
+    if (dsize) {
+	tmp = (*op)(dst->d_buf, src->d_buf, src->d_size);
+	elf_assert(tmp == dsize);
     }
-    dst->d_size = count * dsize;
+    dst->d_size = dsize;
     dst->d_type = type;
     return dst;
 }
