@@ -1,6 +1,6 @@
 /*
 data.c - libelf internal variables.
-Copyright (C) 1995, 1996 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -19,6 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include <private.h>
 
+#ifndef lint
+static const char rcsid[] = "@(#) $Id: data.c,v 1.3 1998/06/12 19:42:19 michael Exp $";
+#endif /* lint */
+
 unsigned _elf_version = EV_NONE;
 int _elf_errno = 0;
 int _elf_fill = 0;
