@@ -1,6 +1,6 @@
 /*
 hash.c - implementation of the elf_hash(3) function.
-Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 2002 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -20,17 +20,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include <private.h>
 
 #ifndef lint
-static const char rcsid[] = "@(#) $Id: hash.c,v 1.3 1998/06/12 19:42:28 michael Exp $";
+static const char rcsid[] = "@(#) $Id: hash.c,v 1.6 2002/12/30 02:10:14 michael Exp $";
 #endif /* lint */
 
 unsigned long
-elf_hash(const char *name) {
+elf_hash(const unsigned char *name) {
     unsigned long hash = 0;
     unsigned long tmp;
-    unsigned char c;
 
-    while ((c = *name++)) {
-	hash = (hash << 4) + c;
+    while (*name) {
+	hash = (hash << 4) + (unsigned char)*name++;
 	if ((tmp = hash & 0xf0000000)) {
 	    hash ^= tmp | (tmp >> 24);
 	}
