@@ -1,6 +1,6 @@
 /*
 newscn.c - implementation of the elf_newscn(3) function.
-Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998, 2003 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -20,9 +20,48 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include <private.h>
 
 #ifndef lint
-static const char rcsid[] = "@(#) $Id: newscn.c,v 1.5 1998/06/12 19:42:32 michael Exp $";
+static const char rcsid[] = "@(#) $Id: newscn.c,v 1.7 2003/05/24 15:51:50 michael Exp $";
 #endif /* lint */
 
+int
+_elf_update_shnum(Elf *elf, size_t shnum) {
+    Elf_Scn *scn;
+
+    if (elf->e_class == ELFCLASS32) {
+	if (shnum >= SHN_LORESERVE) {
+	    scn = elf->e_scn_1;
+	    elf_assert(scn->s_index == 0);
+	    scn->s_shdr_flags |= ELF_F_DIRTY;
+	    scn->s_shdr32.sh_size = shnum;
+	    shnum = 0;
+	}
+	elf->e_ehdr_flags |= ELF_F_DIRTY;
+	((Elf32_Ehdr*)elf->e_ehdr)->e_shnum = shnum;
+	return 0;
+    }
+#if __LIBELF64
+    else if (elf->e_class == ELFCLASS64) {
+	if (shnum >= SHN_LORESERVE) {
+	    scn = elf->e_scn_1;
+	    elf_assert(scn->s_index == 0);
+	    scn->s_shdr_flags |= ELF_F_DIRTY;
+	    scn->s_shdr64.sh_size = shnum;
+	    shnum = 0;
+	}
+	elf->e_ehdr_flags |= ELF_F_DIRTY;
+	((Elf64_Ehdr*)elf->e_ehdr)->e_shnum = shnum;
+	return 0;
+    }
+#endif /* __LIBELF64 */
+    else if (valid_class(elf->e_class)) {
+	seterr(ERROR_UNIMPLEMENTED);
+    }
+    else {
+	seterr(ERROR_UNKNOWN_CLASS);
+    }
+    return -1;
+}
+
 static Elf_Scn*
 _buildscn(Elf *elf) {
     Elf_Scn *scn;
@@ -38,11 +77,19 @@ _buildscn(Elf *elf) {
 	scn->s_freeme = 1;
 	if (elf->e_scn_n) {
 	    scn->s_index = elf->e_scn_n->s_index + 1;
+	    if (_elf_update_shnum(elf, scn->s_index + 1)) {
+		free(scn);
+		return NULL;
+	    }
 	    elf->e_scn_n->s_link = scn;
 	    elf->e_scn_n = scn;
 	    return scn;
 	}
-	elf_assert(scn->s_index == SHN_UNDEF);
+	scn->s_index = 0;
+	if (_elf_update_shnum(elf, 1)) {
+	    free(scn);
+	    return NULL;
+	}
 	elf->e_scn_1 = elf->e_scn_n = scn;
     }
     seterr(ERROR_MEM_SCN);
@@ -66,29 +113,8 @@ elf_newscn(Elf *elf) {
     else if (!elf->e_ehdr && !_elf_cook(elf)) {
 	return NULL;
     }
-    else if (elf->e_class == ELFCLASS32) {
-	if (!(scn = _buildscn(elf))) {
-	    return NULL;
-	}
-	((Elf32_Ehdr*)elf->e_ehdr)->e_shnum = scn->s_index + 1;
-	elf->e_ehdr_flags |= ELF_F_DIRTY;
+    else if ((scn = _buildscn(elf))) {
 	return scn;
     }
-#if __LIBELF64
-    else if (elf->e_class == ELFCLASS64) {
-	if (!(scn = _buildscn(elf))) {
-	    return NULL;
-	}
-	((Elf64_Ehdr*)elf->e_ehdr)->e_shnum = scn->s_index + 1;
-	elf->e_ehdr_flags |= ELF_F_DIRTY;
-	return scn;
-    }
-#endif /* __LIBELF64 */
-    else if (valid_class(elf->e_class)) {
-	seterr(ERROR_UNIMPLEMENTED);
-    }
-    else {
-	seterr(ERROR_UNKNOWN_CLASS);
-    }
     return NULL;
 }
