@@ -1,6 +1,6 @@
 /*
 nlist.h - public header file for nlist(3).
-Copyright (C) 1995, 1996 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -17,12 +17,14 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+/* @(#) $Id: nlist.h,v 1.3 1998/06/01 19:47:24 michael Exp $ */
+
 #ifndef _NLIST_H
 #define _NLIST_H
 
 #ifdef __cplusplus
 extern "C" {
-#endif
+#endif /* __cplusplus */
 
 struct nlist {
     char*		n_name;
@@ -35,12 +37,12 @@ struct nlist {
 
 #if __STDC__ || defined(__cplusplus)
 extern int nlist(const char *__filename, struct nlist *__nl);
-#else
+#else /* __STDC__ || defined(__cplusplus) */
 extern int nlist();
-#endif
+#endif /* __STDC__ || defined(__cplusplus) */
 
 #ifdef __cplusplus
 }
-#endif
+#endif /* __cplusplus */
 
 #endif /* _NLIST_H */
